local VERSION = tonumber(("$Revision: 42560 $"):match("%d+"))

local Parrot = Parrot
local Parrot_Events = Parrot:NewModule("Events", "Parser-3.0", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-02 23:44:17 -0400 (Mon, 02 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot-Events")

local new, del = Parrot.new, Parrot.del

local combatEvents = {}

function Parrot_Events:RegisterCombatEvent(kind, name, localName, defaultTag, event, translation)
	AceLibrary.argCheck(self, kind, 2, "string")
	AceLibrary.argCheck(self, name, 3, "string")
	AceLibrary.argCheck(self, localName, 4, "string")
	AceLibrary.argCheck(self, defaultTag, 5, "string")
	AceLibrary.argCheck(self, event, 6, "table", "string", "nil")
	AceLibrary.argCheck(self, translation, 7, "table", "nil")
	if not combatEvents[kind] then
		combatEvents[kind] = new()
	end
	local t = new()
	combatEvents[kind][name] = t
	t.localName = localName
	t.tag = defaultTag
	t.event = event
	t.translation = translation
	if event then
		if type(event) == "table" then
			self:RegisterParserEvent(event, function(info)
				self:TriggerCombatEvent(kind, name, info.isCrit, info)
			end)
		else
			self:RegisterEvent(event, function(...)
				local info = new(...)
				self:TriggerCombatEvent(kind, name, true, info)
				info = del(info)
			end)
		end
	end
end

local handler__translation
local handler__info
local function handler(literal, inner)
	local value = handler__translation[inner]
	if value then
		if type(value) == "string" then
			return tostring(handler__info[value])
		else
			return tostring(value(handler__info))
		end
	else
		return literal
	end
end

function Parrot_Events:TriggerCombatEvent(kind, name, sticky, info)
	local data = combatEvents[kind]
	if not data then
		return
	end
	data = data[name]
	if not data then
		return
	end
	local text = data.tag
	handler__translation = data.translation
	if handler__translation then
		handler__info = info
		text = text:gsub("(%[([^%]]+)%])", handler)
		handler__translation = nil
		handler__info = nil
	end
	Parrot:ShowMessage(text, kind, sticky)
end


Parrot_Events:RegisterCombatEvent(
	"Notification",
	"Enter combat", L["Enter combat"],
	"+Combat",
	"PLAYER_REGEN_DISABLED"
)

Parrot_Events:RegisterCombatEvent(
	"Notification",
	"Leave combat", L["Leave combat"],
	"-Combat",
	"PLAYER_REGEN_ENABLED"
)

local damageTypeToColor = {
	["Physical"] = "ffffff",
	["Holy"] = "ffff7f",
	["Fire"] = "ff7f7f",
	["Nature"] = "7fff7f",
	["Frost"] = "7f7fff",
	["Shadow"] = "7f007f",
	["Arcane"] = "ff7fff",
}

local coloredDamageAmount = function(info) return ("|cff%s%s|r"):format(damageTypeToColor[info.damageType], info.amount) end

Parrot_Events:RegisterCombatEvent(
	"Incoming",
	"Melee Damage", L["Melee Damage"],
	"([Name]) - [Amount]",
	{
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	{
		Name = "sourceName",
		Amount = coloredDamageAmount,
	}
)

Parrot_Events:RegisterCombatEvent(
	"Outgoing",
	"Melee Damage", L["Melee Damage"],
	"[Amount]",
	{
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	{
		Name = "recipientName",
		Amount = coloredDamageAmount,
	}
)

Parrot_Events:RegisterCombatEvent(
	"Incoming",
	"Skill Damage", L["Skill Damage"],
	"([Name]) - [Amount]",
	{
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	{
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	}
)

Parrot_Events:RegisterCombatEvent(
	"Outgoing",
	"Skill Damage", L["Skill Damage"],
	"[Amount]",
	{
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	{
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	}
)

Parrot_Events:RegisterCombatEvent(
	"Incoming",
	"Skill DoTs", L["Skill DoTs"],
	"([Name]) - [Amount]",
	{
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	{
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	true -- throttle
)

Parrot_Events:RegisterCombatEvent(
	"Outgoing",
	"Skill DoTs", L["Skill DoTs"],
	"[Amount]",
	{
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	{
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	true -- throttle
)
